@@ -31,7 +31,7 @@ from apischema.dependent_required import (
     Requirements,
 )
 from apischema.metadata.implem import ConversionMetadata
-from apischema.metadata.keys import CONVERSIONS_METADATA
+from apischema.metadata.keys import ALIAS_METADATA, CONVERSIONS_METADATA
 from apischema.types import AnyType
 from apischema.typing import get_type_hints, get_type_hints2
 from apischema.utils import (
@@ -43,7 +43,12 @@ from apischema.utils import (
 )
 
 if TYPE_CHECKING:
-    from apischema.conversions.conversions import ResolvedConversion
+    from apischema.conversions.conversions import Conversions
+
+try:
+    from apischema.typing import Annotated
+except ImportError:
+    Annotated = ...  # type: ignore
 
 if sys.version_info <= (3, 7):  # pragma: no cover
     is_dataclass_ = is_dataclass
@@ -117,8 +122,6 @@ def get_default(field: Field) -> Any:
 
 
 def get_alias(field: Field) -> str:
-    from apischema.metadata.keys import ALIAS_METADATA
-
     return field.metadata.get(ALIAS_METADATA, field.name)
 
 
@@ -151,31 +154,15 @@ def check_merged_class(merged_cls: AnyType) -> Type:
     return origin
 
 
-def get_field_conversion(
-    field: Field, field_type: AnyType, operation: OperationKind
-) -> Tuple[AnyType, Optional["ResolvedConversion"]]:
-    from apischema.conversions.fields import (
-        resolve_field_deserialization,
-        resolve_field_serialization,
-    )
+def get_field_conversions(
+    field: Field, operation: OperationKind
+) -> Optional["Conversions"]:
 
     if CONVERSIONS_METADATA not in field.metadata:
-        return field_type, None
-    conversions = field.metadata[CONVERSIONS_METADATA]
-    assert isinstance(conversions, ConversionMetadata)
-    if (
-        operation == OperationKind.DESERIALIZATION
-        and conversions.deserialization is not None
-    ):
-        conversion = resolve_field_deserialization(
-            field_type, conversions.deserialization
-        )
-        return conversion.source, conversion
-    elif (
-        operation == OperationKind.SERIALIZATION
-        and conversions.serialization is not None
-    ):
-        conversion = resolve_field_serialization(field_type, conversions.serialization)
-        return conversion.target, conversion
+        return None
     else:
-        return field_type, None
+        conversions: ConversionMetadata = field.metadata[CONVERSIONS_METADATA]
+        return {
+            OperationKind.DESERIALIZATION: conversions.deserialization,
+            OperationKind.SERIALIZATION: conversions.serialization,
+        }[operation]
