@@ -28,7 +28,6 @@ from apischema import UndefinedType, serialize
 from apischema.aliases import Aliaser
 from apischema.conversions import identity
 from apischema.conversions.conversions import Conversions
-from apischema.conversions.utils import Converter
 from apischema.conversions.visitor import (
     Conv,
     ConversionsVisitor,
@@ -40,7 +39,7 @@ from apischema.conversions.visitor import (
 from apischema.dataclass_utils import (
     get_alias,
     get_default,
-    get_field_conversion,
+    get_field_conversions,
     get_fields,
     is_required,
 )
@@ -54,27 +53,36 @@ from apischema.graphql.resolvers import (
     resolver_parameters,
     resolver_resolve,
 )
-from apischema.json_schema.refs import get_ref, schema_ref
+from apischema.json_schema.refs import check_ref_type, get_ref, schema_ref
 from apischema.json_schema.schema import Schema, get_schema, merge_schema
+from apischema.metadata.implem import ConversionMetadata
 from apischema.metadata.keys import (
+    CONVERSIONS_METADATA,
     MERGED_METADATA,
     PROPERTIES_METADATA,
+    REQUIRED_METADATA,
     SCHEMA_METADATA,
     check_metadata,
+    get_annotated_metadata,
 )
 from apischema.serialization.serialized_methods import ErrorHandler
 from apischema.skip import filter_skipped
 from apischema.types import AnyType, NoneType
-from apischema.typing import get_origin
+from apischema.typing import get_args, get_origin
 from apischema.utils import (
     Undefined,
     get_args2,
     get_origin2,
     is_union_of,
+    sort_by_annotations_position,
     to_camel_case,
-    type_name,
 )
 
+try:
+    from apischema.typing import Annotated
+except ImportError:
+    Annotated = ...  # type: ignore
+
 JsonScalar = graphql.GraphQLScalarType(
     "JSON",
     specified_by_url="http://www.ecma-international.org/publications/files/ECMA-ST/ECMA-404.pdf",  # noqa: E501
@@ -97,10 +105,6 @@ class Nullable(Exception):
     pass
 
 
-def ref_or_name(tp: AnyType) -> str:
-    return get_ref(tp) or type_name(tp)
-
-
 T = TypeVar("T")
 Lazy = Callable[[], T]
 Thunk = Union[Callable[[], T], T]
@@ -120,10 +124,11 @@ class ObjectField:
     name: str
     type: AnyType
     alias: Optional[str] = None
-    converter: Converter = identity
     conversions: Optional[Conversions] = None
     default: Any = graphql.Undefined
-    parameters: Optional[Tuple[Collection[Parameter], Mapping[str, AnyType]]] = None
+    parameters: Optional[
+        Tuple[Collection[Parameter], Mapping[str, AnyType], Mapping[str, Mapping]]
+    ] = None
     resolve: Optional[Callable] = None
     schema: InitVar[Optional[Schema]] = None
     subscribe: Optional[Callable] = None
@@ -131,6 +136,10 @@ class ObjectField:
     description: Optional[str] = field_(init=False, default=None)
 
     def __post_init__(self, schema: Optional[Schema]):
+        if get_origin(self.type) == Annotated:
+            for annotation in reversed(get_args(self.type)[1:]):
+                if isinstance(annotation, Mapping) and SCHEMA_METADATA in annotation:
+                    schema = merge_schema(annotation[SCHEMA_METADATA], schema)
         if schema is not None and schema.annotations is not None:
             object.__setattr__(self, "description", schema.annotations.description)
             if schema.annotations.deprecated is True:
@@ -147,6 +156,15 @@ IdPredicate = Callable[[AnyType], bool]
 UnionRefFactory = Callable[[Sequence[str]], str]
 
 
+def annotated_schema(tp: AnyType) -> Optional[Schema]:
+    schema = None
+    if get_origin(tp) == Annotated:
+        for annotation in get_args(tp)[1:]:
+            if isinstance(annotation, Mapping) and SCHEMA_METADATA in annotation:
+                schema = merge_schema(annotation[SCHEMA_METADATA], schema)
+    return schema
+
+
 class SchemaBuilder(ConversionsVisitor[Conv, Thunk[graphql.GraphQLType]]):
     def __init__(
         self,
@@ -183,13 +201,13 @@ class SchemaBuilder(ConversionsVisitor[Conv, Thunk[graphql.GraphQLType]]):
     ) -> Thunk[graphql.GraphQLType]:
         for annotation in annotations:
             if isinstance(annotation, schema_ref):
-                annotation.check_type(tp)
+                check_ref_type(tp)
                 ref = annotation.ref
                 if not isinstance(ref, str):
                     raise ValueError("Annotated schema_ref can only be str")
                 self._ref = self._ref or ref
-            if isinstance(annotation, Schema):
-                self._schema = merge_schema(annotation, self._schema)
+            if isinstance(annotation, Mapping) and SCHEMA_METADATA in annotation:
+                self._schema = merge_schema(self._schema, annotation[SCHEMA_METADATA])
         return self.visit_with_schema(tp, self._ref, self._schema)
 
     def any(self) -> Thunk[graphql.GraphQLType]:
@@ -202,18 +220,11 @@ class SchemaBuilder(ConversionsVisitor[Conv, Thunk[graphql.GraphQLType]]):
         return lambda: graphql.GraphQLList(exec_thunk(value_thunk))
 
     def _object_field(self, field: Field, field_type: AnyType) -> ObjectField:
-        field_name = field.name
-        field_type, conversion = get_field_conversion(field, field_type, self.operation)
-        converter, conversions = identity, None
-        if conversion is not None:
-            converter = conversion.converter  # type: ignore
-            conversions = conversion.conversions
         return ObjectField(
-            field_name,
+            field.name,
             field_type,
             alias=get_alias(field),
-            converter=converter,
-            conversions=conversions,
+            conversions=get_field_conversions(field, self.operation),
             default=graphql.Undefined if is_required(field) else get_default(field),
             schema=field.metadata.get(SCHEMA_METADATA),
         )
@@ -221,9 +232,8 @@ class SchemaBuilder(ConversionsVisitor[Conv, Thunk[graphql.GraphQLType]]):
     def _visit_merged(
         self, field: Field, field_type: AnyType
     ) -> Thunk[graphql.GraphQLType]:
-        field_type, conversion = get_field_conversion(field, field_type, self.operation)
         return self.visit_with_conversions(
-            field_type, conversion.conversions if conversion is not None else None
+            field_type, get_field_conversions(field, self.operation)
         )
 
     def dataclass(
@@ -233,7 +243,6 @@ class SchemaBuilder(ConversionsVisitor[Conv, Thunk[graphql.GraphQLType]]):
         fields: Sequence[Field],
         init_vars: Sequence[Field],
     ) -> Thunk[graphql.GraphQLType]:
-        types = dict(types)
         object_fields: List[ObjectField] = []
         merged_types: Dict[str, Thunk[graphql.GraphQLType]] = {}
         for field in get_fields(fields, init_vars, self.operation):
@@ -407,9 +416,7 @@ class InputSchemaBuilder(
         elif field.default is not graphql.Undefined:
             try:
                 default = serialize(
-                    field.converter(field.default),
-                    conversions=field.conversions,
-                    aliaser=self.aliaser,
+                    field.default, conversions=field.conversions, aliaser=self.aliaser
                 )
             except Exception:
                 field_type = Optional[field_type]
@@ -488,13 +495,11 @@ class OutputSchemaBuilder(
             resolve = field.resolve
         else:
             field_name, aliaser = field.name, self.aliaser
-            converter, conversions = field.converter, field.conversions
+            conversions = field.conversions
 
             def resolve(obj, _):
                 return partial_serialize(
-                    converter(getattr(obj, field_name)),
-                    aliaser=aliaser,
-                    conversions=conversions,
+                    getattr(obj, field_name), aliaser=aliaser, conversions=conversions
                 )
 
         resolve = self._wrap_resolve(resolve)
@@ -505,16 +510,21 @@ class OutputSchemaBuilder(
         type_thunk = self.visit_with_conversions(field_type, field.conversions)
         args = None
         if field.parameters is not None:
-            parameters, types = field.parameters
+            parameters, types, params_metadata = field.parameters
             args = {}
             for param in parameters:
                 default: Any = graphql.Undefined
                 param_type = types[param.name]
                 if is_union_of(param_type, graphql.GraphQLResolveInfo):
                     break
+                metadata = get_annotated_metadata(param_type)
+                if param.name in params_metadata:
+                    metadata = {**metadata, **params_metadata[param.name]}
+                if REQUIRED_METADATA in metadata:
+                    pass
                 # Don't put `null` default + handle Undefined as None
                 # also https://github.com/python/typing/issues/775
-                if param.default in {None, Undefined}:
+                elif param.default in {None, Undefined}:
                     param_type = Optional[param_type]
                 # param.default == graphql.Undefined means the parameter is required
                 # even if it has a default
@@ -523,12 +533,24 @@ class OutputSchemaBuilder(
                         default = serialize(param.default)
                     except Exception:
                         param_type = Optional[param_type]
+                conversions = metadata.get(
+                    CONVERSIONS_METADATA, ConversionMetadata()
+                ).deserialization
+                arg_type = self.input_builder.visit_with_conversions(
+                    param_type, conversions
+                )
+                description = None
+                if SCHEMA_METADATA in metadata:
+                    schema: Schema = metadata[SCHEMA_METADATA]
+                    if schema.annotations is not None:
+                        description = schema.annotations.description
 
                 def arg_thunk(
-                    arg_thunk=self.input_builder.visit(param_type),
-                    default=default,
+                    arg_type=arg_type, default=default, description=description
                 ) -> graphql.GraphQLArgument:
-                    return graphql.GraphQLArgument(exec_thunk(arg_thunk), default)
+                    return graphql.GraphQLArgument(
+                        exec_thunk(arg_type), default, description
+                    )
 
                 args[self.aliaser(param.name)] = arg_thunk
         return self.aliaser(field.alias or field.name), lambda: graphql.GraphQLField(
@@ -543,18 +565,13 @@ class OutputSchemaBuilder(
     def _visit_merged(
         self, field: Field, field_type: AnyType
     ) -> Thunk[graphql.GraphQLType]:
-        field_type, conversion = get_field_conversion(field, field_type, self.operation)
+        conversions = get_field_conversions(field, self.operation)
         field_name, aliaser = field.name, self.aliaser
-        converter: Converter = identity
-        conversions: Optional[Conversions] = None
-        if conversion is not None:
-            converter = conversion.converter  # type: ignore
-            conversions = conversion.conversions
         get_prev_merged = self._get_merged if self._get_merged is not None else identity
 
         def get_merge(obj):
             return partial_serialize(
-                converter(getattr(get_prev_merged(obj), field_name)),
+                getattr(get_prev_merged(obj), field_name),
                 aliaser=aliaser,
                 conversions=conversions,
             )
@@ -562,10 +579,7 @@ class OutputSchemaBuilder(
         merged_save = self._get_merged
         self._get_merged = get_merge
         try:
-            return self.visit_with_conversions(
-                field_type,
-                conversion.conversions if conversion is not None else None,
-            )
+            return self.visit_with_conversions(field_type, conversions)
         finally:
             self._get_merged = merged_save
 
@@ -582,19 +596,23 @@ class OutputSchemaBuilder(
             if cls.__name__ not in ("Query", "Mutation", "Subscription"):
                 raise
             name, description = cls.__name__, self._description
-        for resolver_name, (resolver, types) in get_resolvers(
+        for resolver_alias, (resolver, types) in get_resolvers(
             self._generic or cls
         ).items():
             resolver_field = ObjectField(
-                resolver_name,
+                resolver.func.__name__,
                 types["return"],
+                alias=resolver_alias,
                 conversions=resolver.conversions,
-                parameters=(resolver.parameters, types),
+                parameters=(resolver.parameters, types, resolver.parameters_metadata),
                 resolve=self._wrap_resolve(
                     resolver_resolve(resolver, types, self.aliaser)
                 ),
             )
             fields_and_resolvers.append(resolver_field)
+        fields_and_resolvers = sort_by_annotations_position(
+            cls, fields_and_resolvers, lambda f: f.name
+        )
         visited_fields = dict(map(self._field, fields_and_resolvers))
 
         def field_thunk() -> graphql.GraphQLFieldMap:
@@ -676,6 +694,7 @@ class Operation(Generic[T]):
     conversions: Optional[Conversions] = None
     schema: Optional[Schema] = None
     error_handler: ErrorHandler = Undefined
+    parameters_metadata: Mapping[str, Mapping] = field_(default_factory=dict)
 
 
 class Query(Operation):
@@ -721,7 +740,12 @@ def operation_resolver(
 
     (*parameters,) = resolver_parameters(operation.function, check_first=True)
     return operation.alias or operation.function.__name__, Resolver(
-        wrapper, operation.conversions, operation.schema, error_handler, parameters
+        wrapper,
+        operation.conversions,
+        operation.schema,
+        error_handler,
+        parameters,
+        operation.parameters_metadata,
     )
 
 
@@ -757,7 +781,11 @@ def graphql_schema(
                     name,
                     resolver_types["return"],
                     conversions=resolver.conversions,
-                    parameters=(resolver.parameters, resolver_types),
+                    parameters=(
+                        resolver.parameters,
+                        resolver_types,
+                        resolver.parameters_metadata,
+                    ),
                     resolve=resolver_resolve(resolver, resolver_types, aliaser),
                     schema=resolver.schema,
                 )
@@ -776,6 +804,7 @@ def graphql_schema(
                 sub_op.schema,
                 subscriber2.error_handler,
                 sub_parameters,
+                sub_op.parameters_metadata,
             )
             sub_types = resolver.types()
             sub_return = sub_types["return"]
@@ -792,6 +821,7 @@ def graphql_schema(
                 sub_op.schema,
                 subscriber2.error_handler,
                 (),
+                {},
             )
             resolve = resolver_resolve(resolver, {}, aliaser)
             subscriber = replace(subscriber2, error_handler=None)
@@ -812,7 +842,7 @@ def graphql_schema(
                 name,
                 sub_return,
                 conversions=sub_op.conversions,
-                parameters=(sub_parameters, sub_types),
+                parameters=(sub_parameters, sub_types, sub_op.parameters_metadata),
                 resolve=resolve,
                 subscribe=subscribe,
                 schema=sub_op.schema,
