@@ -1,5 +1,5 @@
 from contextlib import contextmanager
-from dataclasses import Field
+from dataclasses import Field, dataclass
 from enum import Enum
 from functools import wraps
 from itertools import chain
@@ -35,7 +35,7 @@ from apischema.conversions.visitor import (
 from apischema.dataclass_utils import (
     get_alias,
     get_default,
-    get_field_conversion,
+    get_field_conversions,
     get_fields,
     get_requirements,
     has_default,
@@ -53,7 +53,7 @@ from apischema.json_schema.constraints import (
 )
 from apischema.json_schema.generation.refs import Refs, RefsExtractor
 from apischema.json_schema.patterns import infer_pattern
-from apischema.json_schema.refs import get_ref, schema_ref
+from apischema.json_schema.refs import check_ref_type, get_ref, schema_ref
 from apischema.json_schema.schema import Schema, get_schema, merge_schema
 from apischema.json_schema.types import JsonSchema, JsonType, json_schema
 from apischema.json_schema.versions import JsonSchemaVersion, RefFactory
@@ -66,14 +66,10 @@ from apischema.metadata.keys import (
 from apischema.serialization import serialize
 from apischema.serialization.serialized_methods import get_serialized_methods
 from apischema.skip import filter_skipped
+from apischema.tagged_unions import TAGS_ATTR
 from apischema.types import AnyType, OrderedDict
 from apischema.typing import get_origin
-from apischema.utils import (
-    OperationKind,
-    UndefinedType,
-    get_args2,
-    get_origin2,
-)
+from apischema.utils import UndefinedType, is_union_of, sort_by_annotations_position
 
 constraint_by_type = {
     int: NumberConstraints,
@@ -90,14 +86,22 @@ def with_schema(method: Method) -> Method:
     def wrapper(self: "SchemaBuilder", *args, **kwargs):
         if self._schema is None:
             return method(self, *args, **kwargs)
-        elif self._schema.override:
-            return JsonSchema(**self._schema.as_dict())
-        else:
-            return JsonSchema(method(self, *args, **kwargs), **self._schema.as_dict())
+        schema = self._schema.as_dict()
+        if not self._schema.override:
+            schema = {**method(self, *args, **kwargs), **schema}
+        return JsonSchema(schema)
 
     return cast(Method, wrapper)
 
 
+@dataclass(frozen=True)
+class ObjectField:
+    name: str
+    alias: str
+    required: bool
+    schema: JsonSchema
+
+
 class SchemaBuilder(ConversionsVisitor[Conv, JsonSchema]):
     def __init__(
         self,
@@ -131,7 +135,7 @@ class SchemaBuilder(ConversionsVisitor[Conv, JsonSchema]):
     def annotated(self, tp: AnyType, annotations: Sequence[Any]) -> JsonSchema:
         for annotation in reversed(annotations):
             if isinstance(annotation, schema_ref):
-                annotation.check_type(tp)
+                check_ref_type(tp)
                 if annotation.ref in self.refs:
                     if self._ignore_first_ref:
                         self._ignore_first_ref = False
@@ -141,8 +145,8 @@ class SchemaBuilder(ConversionsVisitor[Conv, JsonSchema]):
                 ref = annotation.ref
                 if not isinstance(ref, str):
                     raise ValueError("Annotated schema_ref can only be str")
-            if isinstance(annotation, Schema):
-                self._merge_schema(annotation)
+            if isinstance(annotation, Mapping) and SCHEMA_METADATA in annotation:
+                self._merge_schema(annotation[SCHEMA_METADATA])
         return self.visit_with_schema(tp, self._schema)
 
     @with_schema
@@ -159,12 +163,8 @@ class SchemaBuilder(ConversionsVisitor[Conv, JsonSchema]):
         )
 
     def visit_field(self, field: Field, field_type: AnyType):
-        field_type, conversion = get_field_conversion(field, field_type, self.operation)
+        conversions = get_field_conversions(field, self.operation)
         converter: Converter = identity
-        sub_conversions: Optional[Conversions] = None
-        if conversion is not None:
-            converter = conversion.converter  # type: ignore
-            sub_conversions = conversion.conversions
         schema: Optional[Schema] = None
         if SCHEMA_METADATA in field.metadata:
             schema = cast(Schema, field.metadata[SCHEMA_METADATA])
@@ -173,14 +173,14 @@ class SchemaBuilder(ConversionsVisitor[Conv, JsonSchema]):
                     raise TypeError("Invalid ... without field default")
                 try:
                     default = serialize(
-                        converter(get_default(field)), conversions=sub_conversions
+                        converter(get_default(field)), conversions=conversions
                     )
                 except Exception:
                     pass
                 else:
                     annotations = replace(schema.annotations, default=default)
                     schema = replace(schema, annotations=annotations)
-        with self._replace_conversions(sub_conversions):
+        with self._replace_conversions(conversions):
             return self.visit_with_schema(field_type, schema)
 
     def _properties_schema(self, field: Field, field_type: AnyType) -> JsonSchema:
@@ -218,23 +218,20 @@ class SchemaBuilder(ConversionsVisitor[Conv, JsonSchema]):
         finally:
             self._ignore_first_ref = ignore_first_ref
 
-    def _serialized_properties(
-        self, cls: Type
-    ) -> Tuple[Mapping[str, JsonSchema], Collection[str]]:
-        properties: Dict[str, JsonSchema] = {}
-        required: List[str] = []
-        if self.operation == OperationKind.SERIALIZATION:
-            for name, (serialized, types) in get_serialized_methods(
-                self._generic or cls
-            ).items():
-                alias = self.aliaser(name)
-                ret = types["return"]
-                with self._replace_conversions(serialized.conversions):
-                    properties[alias] = json_schema(
-                        readOnly=True, **self.visit_with_schema(ret, serialized.schema)
-                    )
-                if get_origin2(ret) != Union or UndefinedType not in get_args2(ret):
-                    required.append(alias)
+    def _update_properties(
+        self, cls: Type, properties: Dict[str, JsonSchema], required: List[str]
+    ):
+        pass
+
+    def _properties(
+        self, cls: Type, fields: Collection[ObjectField]
+    ) -> Tuple[Mapping[str, JsonSchema], Sequence[str]]:
+        properties, required = {}, []
+        for field in fields:
+            alias = field.alias or self.aliaser(field.name)
+            properties[alias] = field.schema
+            if field.required and alias not in required:
+                required.append(alias)
         return properties, required
 
     @with_schema
@@ -247,11 +244,10 @@ class SchemaBuilder(ConversionsVisitor[Conv, JsonSchema]):
     ) -> JsonSchema:
         assert is_dataclass(cls)
         self._check_constraints(ObjectConstraints)
-        properties = {}
-        required: List[str] = []
         merged_schemas: List[JsonSchema] = []
         pattern_properties = {}
         additional_properties: Union[bool, JsonSchema] = settings.additional_properties
+        object_fields: List[ObjectField] = []
         for field in get_fields(fields, init_vars, self.operation):
             metadata = check_metadata(field)
             field_type = types[field.name]
@@ -269,16 +265,16 @@ class SchemaBuilder(ConversionsVisitor[Conv, JsonSchema]):
                     pattern_properties[pattern] = properties_schema
             else:
                 alias = self.aliaser(get_alias(field))
-                properties[alias] = json_schema(
+                field_schema = json_schema(
                     readOnly=not field.init,
                     writeOnly=field in init_vars,
                     **self.visit_field(field, field_type),
                 )
-                if is_required(field):
-                    required.append(alias)
-        serialized_props, serialized_reqs = self._serialized_properties(cls)
-        properties.update(serialized_props)
-        required.extend(serialized_reqs)
+                object_field = ObjectField(
+                    field.name, alias, is_required(field), field_schema
+                )
+                object_fields.append(object_field)
+        properties, required = self._properties(cls, object_fields)
         dependent_required = {
             self.aliaser(get_alias(field)): sorted(
                 self.aliaser(get_alias(req)) for req in required_by
@@ -287,6 +283,14 @@ class SchemaBuilder(ConversionsVisitor[Conv, JsonSchema]):
                 cls, DependentRequired.requiring, self.operation
             ).items()
         }
+        if hasattr(cls, TAGS_ATTR):
+            return json_schema(
+                type=JsonType.OBJECT,
+                oneOf=[
+                    json_schema(properties={prop: value})
+                    for prop, value in properties.items()
+                ],
+            )
         result = json_schema(
             type=JsonType.OBJECT,
             properties=properties,
@@ -359,9 +363,7 @@ class SchemaBuilder(ConversionsVisitor[Conv, JsonSchema]):
         self._check_constraints(ObjectConstraints)
         properties = {self.aliaser(key): self.visit(key) for key, cls in types.items()}
         required = [attr for attr in types if attr not in defaults]
-        serialized_props, serialized_reqs = self._serialized_properties(cls)
-        properties.update(serialized_props)
-        required.extend(serialized_reqs)
+        self._update_properties(cls, properties, required)
         return json_schema(
             type=JsonType.OBJECT,
             properties=properties,
@@ -488,6 +490,25 @@ class SerializationSchemaBuilder(SerializationVisitor, SchemaBuilder):
     class RefsExtractor(SerializationVisitor, RefsExtractor):  # type: ignore
         pass
 
+    def _properties(
+        self, cls: Type, fields: Collection[ObjectField]
+    ) -> Tuple[Mapping[str, JsonSchema], Sequence[str]]:
+        all_fields = list(fields)
+        for name, (serialized, types) in get_serialized_methods(
+            self._generic or cls
+        ).items():
+            ret = types["return"]
+            with self._replace_conversions(serialized.conversions):
+                schema = json_schema(
+                    readOnly=True, **self.visit_with_schema(ret, serialized.schema)
+                )
+            required = not is_union_of(ret, UndefinedType)
+            all_fields.append(
+                ObjectField(serialized.func.__name__, name, required, schema)
+            )
+        all_fields = sort_by_annotations_position(cls, all_fields, lambda f: f.name)
+        return super()._properties(cls, all_fields)
+
     visit_conversion = with_schema(SerializationVisitor.visit_conversion)  # type: ignore # noqa: E501
 
 
@@ -545,8 +566,6 @@ def _schema(
     with_schema: bool,
 ) -> Mapping[str, Any]:
     add_defs = ref_factory is None
-    if ref_factory is not None and all_refs is None:
-        all_refs = True
     if aliaser is None:
         aliaser = settings.aliaser()
     version, ref_factory, all_refs = _default_version(version, ref_factory, all_refs)
