@@ -2,10 +2,12 @@ import collections.abc
 import re
 from enum import Enum, auto
 from functools import wraps
+from inspect import signature
 from types import FunctionType
 from typing import (
     Any,
     Callable,
+    Collection,
     Dict,
     Generic,
     Hashable,
@@ -25,6 +27,7 @@ from apischema.types import (
     AnyType,
     COLLECTION_TYPES,
     MAPPING_TYPES,
+    OrderedDict,
     subscriptable_origin,
 )
 from apischema.typing import _GenericAlias, _collect_type_vars, get_args, get_origin
@@ -69,6 +72,13 @@ def to_hashable(data: Union[None, int, float, str, bool, list, dict]) -> Hashabl
     return data  # type: ignore
 
 
+def contains(collection: Collection[T], obj: T) -> bool:
+    try:
+        return obj in collection
+    except TypeError:
+        return False
+
+
 SNAKE_CASE_REGEX = re.compile(r"_([a-z\d])")
 
 
@@ -76,15 +86,6 @@ def to_camel_case(s: str):
     return SNAKE_CASE_REGEX.sub(lambda m: m.group(1).upper(), s)
 
 
-def type_name(tp: AnyType) -> str:
-    if hasattr(tp, "__name__"):
-        return tp.__name__
-    elif isinstance(tp, _GenericAlias):
-        return tp._name
-    else:
-        raise NotImplementedError
-
-
 MakeDataclassField = Union[Tuple[str, AnyType], Tuple[str, AnyType, Any]]
 
 
@@ -162,7 +163,9 @@ def get_args2(tp: AnyType) -> Tuple[AnyType, ...]:
 
 
 def get_origin_or_type(tp: AnyType) -> Type:
-    origin = get_origin2(tp)
+    origin = get_origin(tp)
+    if origin is Annotated:
+        return get_origin_or_type(get_args(tp)[0])
     return origin if origin is not None else tp
 
 
@@ -202,6 +205,7 @@ def is_method(method: MethodOrProperty) -> bool:
         isinstance(method, FunctionType)
         and method.__name__ != method.__qualname__
         and isinstance(_method_location(method), (type, type(None)))
+        and next(iter(signature(method).parameters), None) == "self"
     )
 
 
@@ -253,6 +257,36 @@ class MethodWrapper(Generic[T]):
     def __set_name__(self, owner, name):
         setattr(owner, name, self._method)
 
+    def __call__(self, *args, **kwargs):
+        raise RuntimeError("Method __set_name__ has not been called")
+
+
+def method_registerer(
+    arg: Optional[Callable],
+    owner: Optional[Type],
+    register: Callable[[Callable, Optional[Type], str], None],
+):
+    def decorator(method: MethodOrProperty):
+        if owner is None and is_method(method) and method_class(method) is None:
+
+            class ResolverDescriptor(MethodWrapper[MethodOrProperty]):
+                def __set_name__(self, owner, name):
+                    super().__set_name__(owner, name)
+                    register(method_wrapper(method), owner, name)
+
+            return ResolverDescriptor(method)
+        else:
+            owner2 = owner
+            if is_method(method):
+                if owner2 is None:
+                    owner2 = method_class(method)
+                method = method_wrapper(method)
+            assert not isinstance(method, property)
+            register(cast(Callable, method), owner2, method.__name__)
+            return method
+
+    return decorator if arg is None else decorator(arg)
+
 
 def replace_builtins(tp: AnyType) -> AnyType:
     origin = get_origin_or_type(tp)
@@ -270,6 +304,16 @@ def replace_builtins(tp: AnyType) -> AnyType:
     return Annotated[(res, *get_args(tp)[1:])] if get_origin(tp) == Annotated else res
 
 
+def sort_by_annotations_position(
+    cls: Type, elts: Collection[T], key: Callable[[T], str]
+) -> List[T]:
+    annotations: Dict[str, Any] = OrderedDict()
+    for base in reversed(cls.__mro__):
+        annotations.update(getattr(base, "__annotations__", ()))
+    positions = {key: i for i, key in enumerate(annotations)}
+    return sorted(elts, key=lambda elt: positions.get(key(elt), len(positions)))
+
+
 try:
     from functools import cached_property
 except ImportError:
